<?php

declare(strict_types=1);

use yii;
use yii\bootstrap\ActiveForm;
use yii\helpers\Html;

use app\models\Notification;
use app\models\Account;
use app\models\Document;
use app\models\Product;
use app\models\Settings;
use app\models\Supply;

use DateTime;
use DateTimeZone;

// Инициализация
$panel ?? $panel = 'profile_panel_input_accounts';
$panel_accounts ?? $panel_accounts = 'profile_panel_input_accounts_control';

// Инициализация часового пояса
preg_match_all('/UTC([\+\-0-9:]*)/', $account->zone ?? Settings::searchActive()['timezone_default'] ?? 'UTC+3', $timezone);
$timezone = $timezone[1][0];
?>

<link href="/css/pages/profile.css" rel="stylesheet">

<div id="page_profile" class="container mb-auto">
    <div class="row h-100 py-3">
        <nav class="col-3">
            <?= $sidebar ?? yii::$app->controller->renderPartial('/profile/sidebar') ?>
        </nav>
        <article class="col-9">
            <div class="p-4 rounded">
                <div id="profile_panel_panel" class="profile_panel">
                    <div class="profile_panel_menu mb-3">
                        <label class="btn button_white mb-0 mr-2" for="profile_panel_input_notifications" onclick="return page_profile_panel_choose('profile_panel_input_notifications');">Уведомления</label>
                        <label class="btn button_white mb-0 mr-2" for="profile_panel_input_accounts" onclick="return page_profile_panel_choose('profile_panel_input_accounts');">Аккаунты</label>
                        <label class="btn button_white mb-0 mr-2" for="profile_panel_input_products" onclick="return page_profile_panel_choose('profile_panel_input_products');">Товары</label>
                        <label class="btn button_white mb-0 mr-2" for="profile_panel_input_supplies" onclick="return page_profile_panel_choose('profile_panel_input_supplies');">Поставки</label>
                        <label class="btn button_white mb-0 mr-2" for="profile_panel_input_settings" onclick="return page_profile_panel_choose('profile_panel_input_settings');">Настройки</label>
                    </div>
                    <div class="profile_panel_content">
                        <input type="radio" id="profile_panel_input_notifications" name="main_panel" <?= $panel === 'profile_panel_input_notifications' ? 'checked' : null ?> />
                        <div class="col">
                            <h5>Отправка уведомления</h5>
                            <div class="dropdown-divider mb-3"></div>
                            <?php
                            $form = ActiveForm::begin([
                                'id' => 'form_profile_panel_notifications',
                                'action' => false,
                                'fieldConfig' => [
                                    'template' => '{label}{input}'
                                ],
                                'options' => [
                                    'onsubmit' => 'return false;'
                                ]
                            ]);

                            // Значения по умолчанию
                            $model_notifications ?? $model_notifications = new Notification;
                            $model_notifications->account ?? $model_notifications->account = null;
                            $model_notifications->text ?? $model_notifications->text = '';
                            ?>

                            <?= $form->errorSummary($model_notifications) ?>

                            <div class="row">
                                <?= $form->field($model_notifications, 'account', ['options' => ['class' => "mb-1 col-9"]])->input('text', ['placeholder' => yii::$app->user->identity->_key]); ?>
                                <?= $form->field($model_notifications, 'type', ['options' => ['class' => "col pl-0"]])->dropDownList($model_notifications->typs); ?>
                            </div>
                            <small class="d-block mb-1"><b>Множественная отправка:</b> @all или перечислить через запятую</small>
                            <small class="d-block mb-3"><b>Тестирование:</b> @test или оставить поле пустым</small>

                            <?= $form->field($model_notifications, 'text', ['options' => ['class' => "mb-1"]])->textarea(["maxlength" => '300']); ?>

                            <p class="mb-3 text-right">
                                <span id="notification-text-current">0</span>
                                <span> / </span>
                                <span id="notification-text-maximum">300</span>
                            </p>

                            <?= Html::submitButton('Отправить', ['name' => 'submitNotification', 'onclick' => 'page_profile_panel_notification_create(this.parentElement);', 'id' => 'notification-button-send', 'class' => 'flex-grow-1 mr-2 btn button_white button_clean']) ?>
                            <?= Html::submitButton('Отправить как HTML', ['name' => 'submitNotification', 'onclick' => 'page_profile_panel_notification_create(this.parentElement, 1);', 'id' => 'notification-button-send-html', 'class' => 'flex-grow-1 mr-2 btn button_white button_clean']) ?>

                            <?php ActiveForm::end(); ?>
                        </div>

                        <input type="radio" id="profile_panel_input_accounts" name="main_panel" <?= $panel === 'profile_panel_input_accounts' ? 'checked' : null ?> />
                        <div class="profile_panel_input_accounts_menu mb-4">
                            <label class="mb-0 mr-2 px-2 py-1 btn button_white_small" for="profile_panel_input_accounts_control" onclick="return page_profile_panel_accounts_choose('profile_panel_input_accounts_control');">Пользователи</label>
                            <label class="mb-0 mr-2 px-2 py-1 btn button_white_small" for="profile_panel_input_accounts_suppliers" onclick="return page_profile_panel_accounts_choose('profile_panel_input_accounts_suppliers');">Поставщики</label>
                        </div>
                        <input type="radio" id="profile_panel_input_accounts_control" name="main_panel_accounts" <?= $panel_accounts === 'profile_panel_input_accounts_control' ? 'checked' : null ?> />
                        <div class="col">
                            <h5>Список пользователей</h5>
                            <div class="dropdown-divider mb-4"></div>
                            <div id="profile_panel_input_accounts_list" class="px-3">
                                <?php
                                // Инициализация счетчика аккаунтов
                                $amount = 0;

                                // Чтение аккаунтов
                                $accounts = Account::read(limit: 100, order: ['desc']);
                                ?>
                                <?php foreach ($accounts ?? [] as $account) : ?>
                                    <?php
                                    foreach ($account->jrnl ?? [] as $jrnl) {
                                        // Перебор записей в журнале

                                        if ($jrnl['action'] === 'create') {
                                            // Найдена дата создания

                                            // Инициализация даты
                                            $create = (new DateTime())->setTimestamp($jrnl['date'])->setTimezone(new DateTimeZone($timezone))->format('d.m.Y') ?? 'Неизвестно';

                                            // Выход из цикла
                                            break;
                                        }
                                    }
                                    ?>

                                    <div class="mb-3 row">
                                        <div class="pr-0 col-auto"><?= ++$amount ?>.</div>
                                        <div class="pr-0 col overflow-hidden" title="ФИО">
                                            <?= $account->name ?? 'Неизвестно' ?>
                                        </div>
                                        <div class="my-auto pr-0 col-auto" title="Псевдоним">
                                            <?= $account->indx ?? '' ?>
                                        </div>
                                        <div class="my-auto pr-0 col-auto" title="Тип аккаунта">
                                            <?= $account->agnt ? 'Поставщик' : 'Покупатель' ?>
                                        </div>
                                        <div class="mr-3 my-auto pr-0 col-2" title="Уровень авторизации">
                                            <?= $account->type() ?>
                                        </div>
                                        <div class="my-auto pr-0 col-auto text-right" title="Дата регистрации">
                                            <?= $create ?? 'Неизвестно' ?>
                                        </div>
                                        <a class="my-auto col-auto fas fa-trash-alt text-dark" type="button" onclick="page_profile_supplies_delete()"></a>
                                    </div>

                                    <?php if ($amount < count($accounts)) : ?>
                                        <div class="dropdown-divider mb-3"></div>
                                    <?php endif ?>
                                <?php endforeach ?>
                            </div>
                        </div>

                        <input type="radio" id="profile_panel_input_accounts_suppliers" name="main_panel_accounts" <?= $panel_accounts === 'profile_panel_input_accounts_suppliers' ? 'checked' : null ?> />
                        <div class="col">
                            <h5>Заявки на регистрацию поставщиков</h5>
                            <div class="dropdown-divider mb-4"></div>
                            <div id="profile_panel_input_suppliers_requests"></div>
                        </div>

                        <input type="radio" id="profile_panel_input_products" name="main_panel" <?= $panel === 'profile_panel_input_products' ? 'checked' : null ?> />
                        <div class="col">
                            <h5>Список товаров</h5>
                            <div class="dropdown-divider mb-4"></div>
                            <?php
                            // Инициализация счетчика аккаунтов
                            $amount = 0;

                            // Чтение аккаунтов
                            $products = Product::read(limit: 100, order: ['desc']);
                            ?>
                            <?php foreach ($products ?? [] as $product) : ?>
                                <?php

                                foreach ($product->jrnl ?? [] as $jrnl) {
                                    // Перебор записей в журнале

                                    if ($jrnl['action'] === 'create') {
                                        // Найдена дата создания

                                        // Инициализация даты
                                        $create = (new DateTime())->setTimestamp($jrnl['date'])->setTimezone(new DateTimeZone($timezone))->format('H:i d.m.Y') ?? 'Неизвестно';

                                        // Выход из цикла
                                        break;
                                    }
                                }
                                ?>

                                <div class="mb-3 row">
                                    <div class="pr-0 col-auto"><?= ++$amount ?>.</div>
                                    <a class="pr-0 col overflow-hidden" title="<?= $product->name ?? 'Открыть страницу товара' ?>" href="/product/<?= $product->catn ?? 'Неизвестно' ?>">
                                        <?= $product->catn ?? 'Неизвестно' ?>
                                    </a>
                                    <div class="my-auto pr-0 col-auto text-right">
                                        <?= $create ?? 'Неизвестно' ?>
                                    </div>
                                    <a class="my-auto col-auto fas fa-trash-alt text-dark" type="button" onclick="page_profile_supplies_delete()"></a>
                                </div>

                                <?php if ($amount < count($products)) : ?>
                                    <div class="dropdown-divider mb-3"></div>
                                <?php endif ?>
                            <?php endforeach ?>
                        </div>

                        <input type="radio" id="profile_panel_input_supplies" name="main_panel" <?= $panel === 'profile_panel_input_supplies' ? 'checked' : null ?> />
                        <div class="col">
                            <h5>Список поставок</h5>
                            <div class="dropdown-divider mb-4"></div>
                            <?php
                            // Инициализация счетчика аккаунтов
                            $amount = 0;

                            // Чтение аккаунтов
                            $supplies = Supply::read(limit: 100, order: ['desc']);
                            ?>
                            <?php foreach ($supplies ?? [] as $supply) : ?>
                                <?php
                                // Поиск аккаунта владельца
                                $account = '/account/' . Account::searchBySupplyId($supply->readId())['_key'];

                                // Деинициализация товара
                                unset($product);

                                if ($connect = $supply->searchConnectWithProduct()) {
                                    // Найдена привязка поставки к аккаунту

                                    $product = Product::searchById($connect->_to);
                                }

                                foreach ($supply->jrnl ?? [] as $jrnl) {
                                    // Перебор записей в журнале

                                    if ($jrnl['action'] === 'create') {
                                        // Найдена дата создания

                                        // Инициализация даты
                                        $create = (new DateTime())->setTimestamp($jrnl['date'])->setTimezone(new DateTimeZone($timezone))->format('H:i d.m.Y') ?? 'Неизвестно';

                                        // Выход из цикла
                                        break;
                                    }
                                }

                                ?>

                                <div class="mb-3 row">
                                    <div class="pr-0 col-auto"><?= ++$amount ?>.</div>
                                    <div class="pr-0 col-2 overflow-hidden" title="Название">
                                        <?= $supply->catn ?? 'Неизвестно' ?>
                                    </div>
                                    <div class="my-auto pr-0 col" title="Описание">
                                        <?= $supply->dscr ?? 'Без описания' ?>
                                    </div>
                                    <div class="my-auto pr-0 col-auto" title="Количество">
                                        <?= $supply->amnt ?? '0' ?>шт
                                    </div>
                                    <div class="my-auto pr-0 col-auto" title="Стоимость">
                                        <?= $supply->cost ?? '0' ?>р
                                    </div>
                                    <div class="my-auto pr-0 col-auto text-right" title="Дата создания">
                                        <?= $create ?? 'Неизвестно' ?>
                                    </div>
                                    <?php if (empty($product)) : ?>
                                        <a class="my-auto pr-0 col-auto fas fa-shopping-basket icon_red" title="Товар отсутствует" type="button" onclick="return profile_panel_input_suppliers_accounts_create_product(this, <?= "'$supply->catn'" ?? null ?>);"></a>
                                    <?php else : ?>
                                        <a class="my-auto pr-0 col-auto fas fa-shopping-basket text-dark" title="Товар" href="<?= "/product/$product->catn" ?>"></a>
                                    <?php endif ?>
                                    <a class="my-auto pr-0 col-auto fas fa-user text-dark" title="Владелец" href="<?= $account ?? '/' ?>"></a>
                                    <a class="my-auto col-auto fas fa-trash-alt text-dark" type="button" title="Удалить" onclick="page_profile_supplies_delete()"></a>
                                </div>

                                <?php if ($amount < count($supplies)) : ?>
                                    <div class="dropdown-divider mb-3"></div>
                                <?php endif ?>
                            <?php endforeach ?>
                        </div>

                        <input type="radio" id="profile_panel_input_settings" name="main_panel" <?= $panel === 'profile_panel_input_settings' ? 'checked' : null ?> />
                        <div class="col">
                            <?php
                            $form = ActiveForm::begin([
                                'id' => 'form_profile_panel_settings_search_period',
                                'action' => false,
                                'fieldConfig' => [
                                    'template' => '{label}{input}',
                                ],
                                'options' => [
                                    'onsubmit' => 'return false;',
                                    'class' => 'mb-4'
                                ]
                            ]);
                            ?>

                            <?= $form->errorSummary($model_settings, ['header' => 'Получены ошибки:']) ?>

                            <?= $form->field($model_settings, 'search_period', ['options' => ['class' => "mb-1"]])->textInput(['value' => $model_settings['search_period'], 'onChange' => 'page_profile_panel_settings(this.parentElement.parentElement, \'profile_panel_input_settings\')']); ?>
                            <small class="d-block mb-1">Время которое надо ждать для повторного поиска в секундах</small>

                            <?php ActiveForm::end(); ?>

                            <?php
                            $form = ActiveForm::begin([
                                'id' => 'form_profile_panel_settings_search_connect_keep',
                                'action' => false,
                                'fieldConfig' => [
                                    'template' => '{label}{input}',
                                ],
                                'options' => [
                                    'onsubmit' => 'return false;'
                                ]
                            ]);

                            $list = ['false' => 'Нет', 'true' => 'Да'];

                            if (isset($model_settings->search_connect_keep)) {
                                // Найден ранее записанный параметр

                                if (isset($list[$model_settings->search_connect_keep])) {
                                    // Совпадение параметра с параметрами списка найдено

                                    // Буфер для сохранения параметра
                                    $buffer = $list[$model_settings->search_connect_keep];

                                    // Удаление параметра
                                    unset($list[$model_settings->search_connect_keep]);

                                    // Сохранение параметра в начале массива
                                    $list = array_merge([$model_settings->search_connect_keep => $buffer], $list);
                                } else {
                                    // Совпадение параметра с параметрами списка не найдено

                                    // Сохранение параметра из данных аккаунта в начале массива
                                    $list = array_merge([$model_settings->search_connect_keep => $model_settings->search_connect_keep], $list);
                                }
                            }
                            ?>

                            <?= $form->field($model_settings, 'search_connect_keep', ['options' => ['class' => "mb-1"]])->dropDownList($list, ['onChange' => 'page_profile_panel_settings(this.parentElement.parentElement, \'profile_panel_input_settings\')']); ?>
                            <small class="d-block mb-1">Удерживать открытое соединение до истечения срока блокировки поиска?</small>
                            <small class="d-block mb-1">При малой задержке позволяет снизить время загрузки страницы, но при большой будет казаться, что сайт завис</small>

                            <?php ActiveForm::end(); ?>
                        </div>
                    </div>
                </div>
            </div>
        </article>
    </div>
</div>

<script src="/js/profile.js" defer></script>
<script src="/js/product.js" defer></script>
<script src="/js/product_panel.js" defer></script>
<script src="/js/profile_panel.js" defer></script>
<script src="/js/textarea.js" defer></script>
<script defer>
    document.addEventListener('DOMContentLoaded', function() {
        // Загружен документ

        // Инициализация активной вкладки
        page_profile_panel_choose('<?= $panel ?? '' ?>');

        // Инициализация панели для ввода текста (тест уведомлений для администратора)
        initTextarea(
            '#notification-text',
            '#notification-text-current',
            '#notification-text-maximum',
            function() {
                let send = document.getElementById('notification-button-send');
                let send_html = document.getElementById('notification-button-send-html');

                send.removeAttribute('onchange');
                send_html.removeAttribute('onchange');

                send.disabled = true;
                send_html.disabled = true;
            },
            function() {
                let send = document.getElementById('notification-button-send');
                let send_html = document.getElementById('notification-button-send-html');

                send.setAttribute('onchange', 'page_profile_panel_notification_create(this.parentElement);');
                send_html.setAttribute('onchange', 'page_profile_panel_notification_create(this.parentElement, 1);');

                send.disabled = false;
                send_html.disabled = false;
            });
    }, false);
</script>
